// Copyright (C) 2003, 2005 Daisuke Arai <darai@users.sourceforge.jp>
// 
// This program is part of Protra.
//
// Protra is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
// 
// $Id: FunctionDefinitionNode.cs,v 1.3 2007-12-27 08:34:39 panacoran Exp $

using System;
using System.Collections;

namespace Protra.Lib.Lang
{
	/// <summary>
	/// ֐`̃m[h\NXłB
	/// </summary>
	public class FunctionDefinitionNode : Node
	{
		/// <summary>
		/// ֐
		/// </summary>
		private string name;
		/// <summary>
		/// ̖ÕXg
		/// </summary>
		private ArrayList nameList = new ArrayList();
		/// <summary>
		/// m[h̃Xg
		/// </summary>
		private ArrayList nodeList = new ArrayList();


		/// <summary>
		/// ͂܂B
		/// </summary>
		/// <exception cref="Protra.Lib.Lang.ParseException">
		/// ɃG[ꍇthrow܂B
		/// </exception>
		/// <param name="context">ReLXg</param>
		public override void Parse(Context context)
		{
			this.ContextToken = (ContextToken)context.Current;

			// ֐̃`FbN
			context.MoveNext();
			ContextToken ct = (ContextToken)context.Current;
			if(! ct.IsFunctionName)
				throw new ParseException("function name expected", this.ContextToken);
			name = ct.ToString();

			// ̃`FbN
			context.MoveNext();
			ct = (ContextToken)context.Current;
			string token = ct.ToString();
			if(token != "(")
			{
				ct = (ContextToken)context.Current;
				token = ct.ToString();
				if(token != ";")
					throw new ParseException("';' expected", this.ContextToken);
			}
			else
			{
				context.MoveNext();
				ct = (ContextToken)context.Current;
				token = ct.ToString();
				if(token == ")")
				{
					context.MoveNext();
					ct = (ContextToken)context.Current;
					token = ct.ToString();
					if(token != ";")
						throw new ParseException("';' expected", this.ContextToken);
				}
				else
				{
					do
					{
						ct = (ContextToken)context.Current;
						if(! ct.IsLocalVariableName)
							throw new ParseException("function argument expected", this.ContextToken);
						nameList.Add(ct.ToString());
						context.MoveNext();
						ct = (ContextToken)context.Current;
						token = ct.ToString();
						if(token == ")")
						{
							context.MoveNext();
							ct = (ContextToken)context.Current;
							token = ct.ToString();
							if(token != ";")
								throw new ParseException("';' expected", this.ContextToken);
							break;
						}
						else if(token != ",")
							throw new ParseException("',' or ')' expected", this.ContextToken);
					}while(context.MoveNext());
				}
			}

			// {̂̃`FbN
			while(context.MoveNext())
			{
				ct = (ContextToken)context.Current;
				token = ct.ToString();
				if(token == "end")
				{
					context.MoveNext();
					ct = (ContextToken)context.Current;
					token = ct.ToString();
					if(token != ";")
						throw new ParseException("';' expected", this.ContextToken);
					return;
				}

				Node node = new StatementNode();
				node.Parse(context);
				nodeList.Add(node);
			}

			throw new ParseException("unexpected EOF was detected", this.ContextToken);
		}

		/// <summary>
		/// \؂ȗ܂B
		/// </summary>
		/// <returns>ȗꂽm[h</returns>
		public override Node Simplify()
		{
			for(int i = 0; i < nodeList.Count; i++)
			{
				Node node = (Node)nodeList[i];
				nodeList[i] = node.Simplify();
			}
			return this;
		}

		/// <summary>
		/// vOs܂B
		/// </summary>
		/// <exception cref="Protra.Lib.Lang.RuntimeException">
		/// vOsɃG[ꍇthrow܂B
		/// </exception>
		/// <param name="resource">\[X</param>
		/// <param name="at">@pf̒l</param>
		/// <returns>s</returns>
		public override Value Execute(Resource resource, int at)
		{
			// ֐e[uɓo^
			FunctionType ft = new FunctionType(name, nameList.Count);
			if(resource.FunctionTable.ContainsFunctionType(ft))
				throw new RuntimeException("function is already defined --- " + ft.ToString(), this.ContextToken);
			resource.FunctionTable[ft] = this;
			
			return null;
		}

		/// <summary>
		/// ̖ÕXg擾܂B
		/// </summary>
		public string[] ArgumentNames
		{
			get { return (string[])nameList.ToArray(typeof(string)); }
		}

		/// <summary>
		/// m[h̃Xg擾܂B
		/// </summary>
		public Node[] Nodes
		{
			get { return (Node[])nodeList.ToArray(typeof(Node)); }
		}
	}
}
